--[[
NoSpam
Inspired by LFGFilter

manages all channels
If you edit your chat channel setup, you may need to do a /script ReloadUI() for this mod to work.


MustMatch: channels with any filters suppress all messages that dont match a filter
Duplicate: the number of minutes to forbid a duplicate post in that channel

2do
Gag: if a player exceeds the max number of messages in any 5 minute period, they are gagged until they havent spoken for gaggedformins.
whisper: if a player exceeds maxmessages in 5 minutes they get a warning to slow up and you dont see squat.
         If more than maxthreads people try to contact you, they get warned off and you dont see squat.
--]]



------------------------------------------------------
-- API
------------------------------------------------------
local NoSpam_OnLoad, NoSpam_OnEvent



------------------------------------------------------
-- 118N
------------------------------------------------------


NoSpam_Filters ={                     
	MustMatch = {
		Trade = {
			LogPassedMsgs = true,
			LogFailedMsgs = false,
			Filters = {
				"wt[sbt]",
				"sell",
				"buy",
				"trade",
				"%d+[gs]",
				"hitem%:",
				"gold",
				"silver",
				"pric",
				"charg",
				"chant",
				"smith",
				"alch",
				"work",
				"eng",
				"tailor",
			},
	   },
	   LookingForGroup = {
			LogPassedMsgs = false,
			LogFailedMsgs = false,
			Filters = {
				"lf[mg%d ]",
				"king for",
				"scholo",
				"strat",
				"tank",
				"healer",
				"mage",
				"dps",
				" cc",
				"ulda",
				"wc",
				"rfc",
				"brd",
				"depths",
				"wailing",
				"chasm",
				" sm",
				"scarlet",
				"maur",
				"gnomer",
				"sunken",
				"key",
				"dragon",
				"quest",
			},
		},
	},
	Duplicate = {
		Trade = 15,
		LFG = 5,
		General = 15,
		LocalDefense = 5,
		WorldDefense = 5,
		GuildRecruitment = 30,
	},
	Gag = {
	   MaxMsgsIn5Mins = 5,
	},
	Whisper = {
	   MaxMsgs = 30,
	   MaxThreads = 3,
	},
   Spammer = {
		Trade = 5,
		General = 8,
		GuildRecruitment = 8,
	},
}     -- saved


local Trade = "Trade"
local SquareKet = "%]"
local slashcmd1 = "/nsp"
local slashcmd2 = "/nospam"
local NoSpam_LOADED_TEXT = "NoSpam by Slott Industries is loaded. Enter "..slashcmd1.." or "..slashcmd2.." for help.";
local NoSpam_DISABLED_TEXT = "NoSpam is now Off. Eggs with that spam?"

local NoSpam_Buffer = {
	["Duplicate"] = {},
	["Gag"] = {},
	["Whisper"] = {},
	["Spammer"] = {},
}



------------------------------------------------------
-- other vars
------------------------------------------------------

local Dunno = 0;
local SendIt = 1;
local DontSend = 2;

NoSpam_Logs = {}

local NoSpam_IsEnabled = false

local NoSpam_Fade = DEFAULT_CHAT_FRAME:GetFadeDuration()



local realm, faction



------------------------------------------------------
-- Init
------------------------------------------------------

function NoSpam_OnLoad()
   SLASH_NOSPAM1 = slashcmd1;
   SLASH_NOSPAM2 = slashcmd2;
   
   SlashCmdList["NOSPAM"] = NoSpam_Commands;

	this:RegisterEvent("PLAYER_LOGIN")
end



function NoSpam_OnEvent(event, ...)
	if event=="PLAYER_LOGIN" then -- init
 		realm = GetRealmName()
		faction = string.sub(UnitFactionGroup("player"), 1, 1)
	   NoSpam_Hook();
	 	NoSpam_Init()
	 	NoSpam_Enable();
	end
end



function NoSpam_Init()
	-- Init all vars as required by config
	
	
	-- MustMatch - set up logs
	if NoSpam_Filters["MustMatch"] ~= nil then -- we have some channels that may need logging

		for channel,settings in pairs(NoSpam_Filters["MustMatch"]) do
      	if NoSpam_Filters["MustMatch"][channel]["LogPassedMsgs"] then -- we need a log
				NoSpam_CreateLogTree(channel, "Passed")
			end
      	if NoSpam_Filters["MustMatch"][channel]["LogFailedMsgs"] then -- we need a log
				NoSpam_CreateLogTree(channel, "Failed")
			end
		end

	end
	

	if NoSpam_Filters["Duplicate"] ~= nil then -- make space in the dupes buffer

		for channel,_ in pairs(NoSpam_Filters["Duplicate"]) do
      	NoSpam_Buffer["Duplicate"][channel]={}
		end

	end

	
end



function NoSpam_CreateLogTree(channel, msgType)
	if NoSpam_Logs[channel] == nil then NoSpam_Logs[channel] = {} end
	if NoSpam_Logs[channel][msgType] == nil then NoSpam_Logs[channel][msgType] = {} end
	if NoSpam_Logs[channel][msgType][realm.."-"..faction] == nil then NoSpam_Logs[channel][msgType][realm.."-"..faction] = {} end
end



function NoSpam_WriteLog(channel, msgType, msg)
	NoSpam_Debug("Logging:- Ch:"..channel.."  type:"..msgType.."  msg:"..msg)
	tinsert(NoSpam_Logs[channel][msgType][realm.."-"..faction],msg)
end



function NoSpam_Hook()
-- hook into the addmessage method.  Only call this once!
   if (NoSpam_Filters["MustMatch"] ~= Nil) then NoSpam_Hook_Pairs(NoSpam_Filters["MustMatch"]); end
 	if (NoSpam_Filters["Duplicate"] ~= Nil) then NoSpam_Hook_Pairs(NoSpam_Filters["Duplicate"]); end
 	if (NoSpam_Filters["Gag"] ~= Nil) then NoSpam_Hook_Pairs(NoSpam_Filters["Gag"]["Channels"]); end
 	if (NoSpam_Filters["Whisper"] ~= Nil) then NoSpam_Hook_Whisper(); end
 	if (NoSpam_Filters["Spammer"] ~= Nil) then NoSpam_Hook_Pairs(NoSpam_Filters["Spammer"]); end
end




function NoSpam_Hook_Whisper()
-- uses a special event

end


function NoSpam_Hook_Pairs(Source)
	if (Source == nil) then return; end

	for channel,filtersettings in pairs(Source) do
		local NoSpam_HostFrame = ChatFrameWithChannel(channel)
		if (NoSpam_HostFrame ~= nil) and (NoSpam_HostFrame.NoSpam_Orig_AddMessage == nil) then
			NoSpam_HostFrame.NoSpam_Orig_AddMessage = NoSpam_HostFrame.AddMessage
	   	NoSpam_HostFrame.AddMessage= NoSpam_AddMessage
		end
	end
end



------------------------------------------------------
-- Main functions
------------------------------------------------------

function NoSpam_AddMessage( frame, OrigMsg, ... )
-- Called on any chat event for the first chat frame of every filtered channel


   -- Instant exits
   
	if ( OrigMsg == nil ) then return; end

 	if not NoSpam_IsEnabled then
		return frame:NoSpam_Orig_AddMessage( OrigMsg, unpack(arg) );
	end


	-- Init
	

	local status = Dunno;
	local channel, msg
	local OrigMsgOnly = msg


	channel, msg = MessageChannel(OrigMsg);
	
	-- No channel is whisper / say / guild
	
	if channel == nil or channel == "" then
     return frame:NoSpam_Orig_AddMessage( OrigMsg, unpack(arg) )
	end
	

	if msg == nil or msg == "" then
     return frame:NoSpam_Orig_AddMessage( OrigMsg, unpack(arg) )
	end	

 	NoSpam_Debug("Ch: "..channel.."  Msg: "..msg)

 	msg = string.lower(msg) -- for test purposes


	-- MustMatch

	status = NoSpam_MustMatch(channel,msg)

---[[

	-- Duplicates
	
	if (status ~= DontSend) then status = NoSpam_Duplicate(channel, msg) end


	--Gag
	
   if (status ~= DontSend) and string.find("LookingForGroup|General|Trade|LocalDefense|GuildRecruitment",channel) then
	   status = NoSpam_Gag(msg)
	end
--]]
	-- Complete!
	
	if (status ~= DontSend) then
		NoSpam_Debug("Passed")
		return frame:NoSpam_Orig_AddMessage( OrigMsg, unpack(arg) );
	end
end



function NoSpam_Gag(msg)
	NoSpam_Debug("Entering gag")
	if NoSpam_Filters["Gag"] ~= nil then -- gag filters are in use

		playername, _ = PlayerMessage(msg)
		if playername == UnitName("player") then return SendIt end

		if not NoSpam_Buffer["Gag"][playername] then NoSpam_Buffer["Gag"][playername]={} end
		tinsert(NoSpam_Buffer["Gag"][playername],time())
		local BufferedTime = time()-300

		_,oldtime = NoSpam_Buffer["Gag"][playername][1]
		if oldtime~=nil then
			while oldtime < BufferedTime do
			   table.remove(NoSpam_Buffer["Gag"][playername],1)
			   _,oldtime = NoSpam_Buffer["Gag"][playername][1]
			end
		end

		if table.getn(NoSpam_Buffer["Gag"][playername])> NoSpam_Filters.Gag.MaxMsgsIn5Mins then
   		return DontSend
		else
		   return SendIt
		end

--]]
	end -- gag filters in use
end



function NoSpam_Duplicate(channel, msg)
--remove buffer entries older than keep time
--find msg in buffer
--if it doesnt exist, add it to buffer
--if it does, don't send the message

	local DupeTimer = NoSpam_Filters["Duplicate"][channel]
	local IsBuffered = false
	
	if  DupeTimer~= nil then -- there is a dupe setting for this channel, filter this message
	
	   DupeTimer = DupeTimer * 60 --mins to secs
	   local BufferTime = time() - DupeTimer
	   
	   --remove old buffer entries
	   if table.getn(NoSpam_Buffer["Duplicate"][channel]) > 0 then
	   
		   msgTime,_ = NoSpam_Buffer["Duplicate"][channel][1]
		   while msgtime < BufferTime do
		      table.remove(NoSpam_Buffer["Duplicate"][channel],1)
		      msgTime,_ = NoSpam_Buffer["Duplicate"][channel][1]
		   end
	   end
	   
	   
		--see if message is buffered
		for msgTime,oldmsg in NoSpam_Buffer["Duplicate"][channel] do
		   if oldmsg == msg then --don't send it, it is a duplicate
		      return DontSend
			end
		end
		
		--message isnt buffered, buffer it
		NoSpam_Buffer["Duplicate"][channel][time()]=msg
		return SendIt
		
	end -- filter this message
end




function NoSpam_MustMatch(channel, msg)

    	NoSpam_Debug("Testing MustMatch")

		if ( NoSpam_Filters["MustMatch"][channel] ~= nil ) then

	    	NoSpam_Debug("This channel has filters")

			if NoSpam_MessageMatchesAFilter(msg, channel) then

	         if NoSpam_Filters["MustMatch"][channel]["LogPassedMsgs"] then
					NoSpam_WriteLog(channel,"Passed", msg)
				end
				return SendIt

			else

	         NoSpam_Debug("did not match a filter")

				if NoSpam_Filters["MustMatch"][channel]["LogFailedMsgs"] then
					NoSpam_WriteLog(channel,"Failed", msg)
				end
			   return DontSend

			end

		else
		   NoSpam_Debug("No must match filters for this channel.")
		   return Dunno
		end

end





function NoSpam_Enable()
    NoSpam_IsEnabled = true
    NoSpam_SendMessage(NoSpam_LOADED_TEXT)
end


function NoSpam_Disable()
    NoSpam_IsEnabled = false
    NoSpam_SendMessage( NoSpam_DISABLED_TEXT )
end



-- handle slash commands.

function NoSpam_Commands(cmd, ...)
    if ( string.find( cmd, "^list" ) ~= nil ) then
        NoSpam_ViewPatterns()
        
    elseif (string.find( cmd, "^add" ) ~= nil ) then
        NoSpam_AddPattern( string.sub( cmd, 5 ))
        
    elseif ( string.find( cmd, "^remove" ) ~= nil ) then
        NoSpam_RemovePattern( string.sub( cmd, 8 ))
        
    elseif ( string.find( cmd, "^off" ) ~= nil ) then
        NoSpam_Disable()
        
    elseif ( string.find( cmd, "^on" ) ~= nil ) then
        NoSpam_Enable()
        
    elseif ( string.find( cmd, "^default" ) ~= nil ) then
        NoSpam_RestoreDefaults()
        
    else
        NoSpam_Help()
        
    end
end


function NoSpam_Help()
    NoSpam_SendMessage( "_____________________________________________________________________")
    NoSpam_SendMessage( "." )
    NoSpam_SendMessage( ".  NoSpam                                                                                        by Slott Industries")
    NoSpam_SendMessage( "_____________________________________________________________________")
    NoSpam_SendMessage( "." )
    NoSpam_SendMessage( ".  Savagely filters your channels. Adjust settings by editing NoSpam.lua" )
    NoSpam_SendMessage( "." )
    NoSpam_SendMessage( ".  /nospam off                                             turn filtering off" )
    NoSpam_SendMessage( ".  /nospam on                                             turn filtering on" )
    NoSpam_SendMessage( "_____________________________________________________________________" )
    NoSpam_SendMessage( "." )
end


-- test the message against the patterns, and if it matches, return true.



function NoSpam_MessageMatchesAFilter( msg, channelname )
	for i,filter in pairs(NoSpam_Filters["MustMatch"][channelname]["Filters"]) do
	
		if string.find( msg, filter ) then
			NoSpam_Debug("Filter passed: "..filter)
			return true
		end
	   NoSpam_Debug("Filter failed: "..filter)
		
	end
	return false
end



function MessageChannel( OrigMsg )
	local channelend = string.find( OrigMsg, "%]" )
	local channelstart = string.find( OrigMsg, " " )
	if (channelend == nil) then return; end

	local msg = strsub(OrigMsg, channelend + 1)
	local channel = strsub(OrigMsg, channelstart + 1, channelend - 1)

	return channel, msg
end



function PlayerMessage(Msg)
-- extracts player name from msg

	if Msg~=nil then
		local player, newmsg
		local playerend, playerstart, newmsgstart
		
--      NoSpam_Debug("extract player from: "..Msg)

		_, playerstart = string.find(Msg,"hplayer:")
		
--		NoSpam_Debug("Start: "..playerstart)
		
		if playerstart ~= nil then --extract playername
		
			playerstart = playerstart + 1
			playerend = string.find(Msg, "|h", playerstart)

         playerend = playerend - 1
			
--			NoSpam_Debug("End: "..playerend)
			
			player = strsub(Msg, playerstart, playerend)
			
			
			_, newmsgstart = string.find(Msg, "|h: ") 
			newmsg = strsub(Msg, newmsgstart + 1,0)

--			NoSpam_Debug("Player: "..player.."   msg: "..newmsg)
			return player,newmsg


		else
			return "",msg
		end
	end
end





-- send a message to the default chat frame.
function NoSpam_SendMessage(msg)
  	if DEFAULT_CHAT_FRAME.NoSpam_Orig_AddMessage ~= nil then
  		DEFAULT_CHAT_FRAME:NoSpam_Orig_AddMessage("|CFF000000"..msg.."|r", 0.0, 0.0, 0.0, 1.0, NoSpam_Fade)
  	else
  		DEFAULT_CHAT_FRAME:AddMessage("|CFF000000"..msg.."|r", 0.0, 0.0, 0.0, 1.0, NoSpam_Fade)
  	end
end


-- set the false to true to get debug messages.
function NoSpam_Debug( msg )
	if ( false ) then
  	   NoSpam_SendMessage( msg)
	end
end




--------------- Utilities  --------------------

function ChatFrameWithChannel(channelName)
	local i, localChannels, name, zone
	
	for i=1,NUM_CHAT_WINDOWS do
	
	   localChannels = {GetChatWindowChannels(i)}

	   for zone,name in pairs(localChannels) do
			if name == channelName then
				return getglobal( "ChatFrame"..i)
			end
		end

	end -- chat frame
end




-------------------

NoSpam = {
	OnLoad = NoSpam_OnLoad,
	OnEvent = NoSpam_OnEvent,
}



--[[
-- add a pattern to our global pattern list.
function NoSpam_AddPattern( pattern )
    tinsert( NoSpam_PATTERN_LIST, pattern )
    NoSpam_SendMessage( pattern.." added" )
end


-- remove a pattern by index from our global pattern list.
function NoSpam_RemovePattern( num )
    local realnum = tonumber( num )
    if ( realnum ~= nil ) and ( realnum > 0 ) and ( realnum <= getn(NoSpam_PATTERN_LIST)) then
        NoSpam_SendMessage( "removing pattern "..NoSpam_PATTERN_LIST[realnum] )
        tremove( NoSpam_PATTERN_LIST, num )
    else
        NoSpam_SendMessage( "unable to remove pattern at index "..num )
    end
end


-- view all patterns in our pattern list.
function NoSpam_ViewPatterns()
    NoSpam_SendMessage( "NoSpam Patterns:" )
    foreachi( NoSpam_PATTERN_LIST, function(k,v) NoSpam_SendMessage( k.." - "..v ) end )
end


-- restore default patterns.
function NoSpam_RestoreDefaults()
   NoSpam_SendMessage( "NoSpam_ patterns restored to default." )
   NoSpam_PATTERN_LIST = NoSpam_PATTERN_LIST_DEFAULTS
end                                                             


--]]